%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                                                                %
%          MatLab function for estimating a restricted VEC       %
%          model for endogenous, exogenous, and deterministic    %
%          variables                                             %
%                                                                %
%          Function: gammarestrict                               %
%                                                                %
%          Input:                                                %
%                       path: text string with path to the       %
%                           Q.txt files with restrictions        %
%                           which is used if file exists         %
%                           (can be a string with a space)       %
%                       Z0: nxT containing all endogenous        %
%                            variables in first diffs            %
%                       Z1: n+q1xT containing first lag of       %
%                            all endogenous and I(1) exogenous   %
%                            variables in levels                 %
%                       Z2: n(p-1)+qp+dxT containing             %
%                            lags of endogenous first diffs,     %
%                            contemporaneous and lags of first   %
%                            diffs of I(1) and levels of I(0)    %
%                            exogenous variables, and            %
%                            deterministic variables             %
%          		alpha: nxr matrix                        %
%          		beta: n+q1xr matrix                      %
%                       Psi: nx(n(p-1)+d) matrix                 %
%                       Omega: nxn matrix                        %
%                       Sigma_pi: n(n(p-1)+r)xn(n(p-1)+r)        %
%                           matrix covariance matrix for         %
%                           alpha and Psi                        %
%                       epsilon: nxT matrix with residuals       %
%                       T: sample size                           %
%                                                                %
%          Output:                                               %
%          		alpha: nxr matrix                        %
%                       Psi: nx(n(p-1)+d) matrix                 %
%                       Omega: nxn matrix                        %
%                       Sigma_pi: n(n(p-1)+r)xn(n(p-1)+r)        %
%                           matrix covariance matrix for         %
%                           alpha and Psi                        %
%                       epsilon: nxT matrix with residuals       %
%                                                                %
%          First version: 	November 14, 2001                %
%          Written by: 		Anders Warne                     %
%          			Copyright (c) 2001               %
%                                                                %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


function [alpha_n,Psi_n,Omega_n,Sigma_pi_n,epsilon_n]=gammarestrict(path,Z0,Z1,Z2,alpha,beta,Psi,Sigma_pi,Omega,epsilon,T)

gamma = vec([Psi alpha]);
n = size(alpha,1);
r = size(alpha,2);

%
% determine the matrix Q with restrictions on gamma
%
% if Q.txt exists and has the correct number of columns,
% i.e. the length of gamma, and has full row rank then this Q
% matrix is used for the restrictions. Else Q is set up from
% such that those elements in gamma with t-values less than
% unity (in absolute terms) are restricted to zero
%

nofile = 1;
file = strcat(path,'Q.txt');
fid = fopen(file,'rt');
if fid>-1;
   Q = load(file);
   if size(Q,2)==size(gamma,1);
      if abs(det(Q*Q'))>0;
         nofile==0;
         numrest = size(Q,1);
         file2 = strcat(path,'qvec.txt');
         fid2 = fopen(file2,'rt');
         if fid>-1;
            qvec = load(file2);
         else;
            qvec = zeros(numrest,1);
         end;
      end;
   end;
end;

if nofile==1;
   tvalues = sqrt(diag(Sigma_pi));
   tvalues = abs(gamma./tvalues);
   Q = zeros(size(gamma,1),size(gamma,1));
   i = 1;
   numrest = 1;
   while i<=size(gamma,1);
      if tvalues(i,1)<1;
         Q(numrest,i) = 1;
         numrest = numrest+1;
      end;
      i = i+1;
   end;
   if numrest>1;
      numrest = numrest-1;
      Q = Q(1:numrest,:);
      qvec = zeros(numrest,1);
      disp(['Estimating short run dynamics under ' num2str(numrest) ' restrictions based']);
      disp('on the unrestricted parameter estimates with t-ratios less');
      disp('than unity in absolute terms');
      disp(' ');
   end;
end;

if numrest==0;
   alpha_n = alpha;
   Psi_n = Psi;
   Omega_n = Omega;
   Sigma_pi_n = Sigma_pi;
   epsilon_n = epsilon;
else;
   Cov_Q = Q*Sigma_pi*Q';
   gamma_n = gamma-(Sigma_pi*Q'*inv(Cov_Q)*(Q*gamma-qvec));
   num = size(gamma,1)/n;
   Phi_fix = vecinv(gamma_n,n);
   Psi_n = Phi_fix(:,1:num-r);
   alpha_n = Phi_fix(:,num-r+1:num);
   epsilon_n = Z0-alpha_n*beta'*Z1-Psi_n*Z2;
   Omega_n = (1/T)*epsilon_n*epsilon_n';
   Sigma_pi_n = Sigma_pi-(Sigma_pi*Q'*inv(Cov_Q)*Q*Sigma_pi);
   Lmax_u = -T*log(det(Omega));
   Lmax_r = -T*log(det(Omega_n));
   LR_Q = 2*(Lmax_u-Lmax_r);

   %--------------------------------------------------------------
   %
   %  Open output file
   %

   diary on;

   txt = strcat(num2str(numrest),' degrees of freedom is:');
   txt = ['The LR test with ' txt];
   disp(txt);
   disp(LR_Q);
   
   fid = fopen('chi2cdf.m','rt');
   if fid>-1;
      disp('with p-value equal to:');
      disp(' ');
      pval = chi2cdf(LR_Q,numrest);
      disp(1-pval);
   end;

   diary off;

   %
   %  Close output file
   %
   %--------------------------------------------------------------

end;

%
% end of function gammarestrict
%



