%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                                                                %
%          MatLab function for computing parameter constancy     %
%          test against LSTR1 model - see Tersvirta (1998)      %
%          "Modelling economic relationships with smooth         %
%          transition regressions", Handbook of Applied          %
%          Statistics, eq. (17) and (21). 1st and 3rd order      %
%          Taylor expansions are considered                      %
%                                                                %
%          Function: lstr1                                       %
%                                                                %
%          Input:                                                %
%                        epsilon: nxT matrix with residuals      %
%                        Z2: n(p-1)+qp+dxT matrix containing     %
%                            lags of endogenous first diffs,     %
%                            contemporaneous and lags of first   %
%                            diffs of I(1) and levels of I(0)    %
%                            exogenous variables, and            %
%                            deterministic variables             %
%                        Zb: rxT matrix with cointegration       %
%                            relations                           %
%                        p: number of lags                       %
%                        d: number of deterministic variables    %
%                        df: number of degrees of freedom        %
%                            per equation in the model           %
%                        ind: index which is x if time           %
%                            trend is element x of Z2 and 0      %
%                            (default) otherwise                 %
%                                                                %
%          Output:                                               %
%                        lmvec: matrix with LM tests + p-values  %
%                        fvec: matrix with F tests + p-values    %
%                        dfmat: matrix with number of            %
%                            degrees of freedom                  %
%                        nrvec: 1x4 vector indicating if a       %
%                            regressor group is available or not %
%                        nof: 1x4 vector indicating if a set     %
%                            of F tests have been skipped due    %
%                            to singularity problems             %
%                        warn: indicator reporting number of     %
%                            deterministic variables which have  %
%                            been removed for these tests (i.e.  %
%                            dummies which are 1 once and 0      %
%                            otherwise and time trends) - these  %
%                            will lead to singularities for the  %
%                            F test                              %
%                                                                %
%          This version:        October 4, 2003                  %
%          Written by: 	        Anders Warne                     %
%                               Copyright (c) 2001-2003          %
%                                                                %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function [lmvec,fvec,dfmat,nrvec,nof,warn] = lstr1(epsilon,Z2,Zb,p,d,df,ind)

%
% preliminaries
%

[n T] = size(epsilon);
Omega = (1/T)*epsilon*epsilon';
trend = 1:1:T;
trend = trend-round(T/2);
trend2 = trend.*trend;
trend3 = trend2.*trend;

warn = 0;
nf1 = 1;
nf2 = 1;
nf3 = 1;
nf4 = 1;
%
% Setting up various regressor matrices
%
x1 = Z2(1:d,:);
if ind>0;
   if ind==1;
      x1 = x1(2:d,:);
   elseif ind==d;
      x1 = x1(1:d-1,:);
   else;
      x1 = [x1(1:ind-1,:);x1(ind+1:d,:)];
   end;
   warn = 1;
end;

nr1 = size(x1,1);

num=1;

if p>1;
   x2 = Z2(d+1:(n*(p-1))+d,:);
   nr2 = size(x2,1);
   num = num+1;
else;
   x2 = 0;
   nr2 = 0;
end;

nr3 = size(Z2,1)-((n*(p-1))+d);
if nr3>0;
   x3 = Z2((n*(p-1))+d+1:size(Z2,1),:);
   num = num+1;
else;
   x3 = 0;
end;

if Zb==0;
   nr4 = 0;
else;
   x4 = Zb;
   nr4 = size(x4,1);
   num = num+1;
end;

lmvec = zeros(num*n,2);
fvec = zeros(num*n,2);
dfmat = zeros(num*n,6);
%
% check if x1 has any dummy which is 1 in one period
% only and 0 otherwise
%
d = size(x1,1);
checkx1 = zeros(d,1);
for i=1:d;
   dtest = sort(x1(i,:));
   if dtest(1,size(dtest,2)-1)==0;
      checkx1(i,1) = 1;
      warn = warn+1;
   end;
end;
if warn>0;
   i = d;
   while i>0;
      if checkx1(i,1)==1;
         if i==1;
            x1 = x1(2:d,:);
         elseif i==d;
            x1 = x1(1:d-1,:);
         else;
            x1 = [x1(1:i-1,:);x1(i+1:d,:)];
         end;
      end;
      i = i-1;
      d = size(x1,1);
      if d==0;
         i = 0;
      end;
   end;
end;
nr1 = size(x1,1);
%
% Test to make sure that degrees of freedom are
% sufficient
%
if df-nr1<=10;
   nr1 = 0;
end;
if df-nr2<=10;
   nr2 = 0;
end;
if df-nr3<=10;
   nr3 = 0;
end;
if df-nr4<=10;
   nr4 = 0;
end;
%
%
%
if nr1>0;
   w1 = x1.*(ones(nr1,1)*trend);
   if df-(3*nr1)>10;
      w12 = x1.*(ones(nr1,1)*trend2);
      w13 = x1.*(ones(nr1,1)*trend3);
      z1 = [w1;w12;w13];
      checkz = min(abs(eig(z1*z1')));
      if checkz>1.0e-08;
         nf1 = 0;
      end;
      if nf1==0;
         b1 = (epsilon*z1')/(z1*z1');
         v1 = epsilon-(b1*z1);
         Omega1 = (1/T)*v1*v1';
      end;
   end;
   M00_1 = x1*x1';
   M01_1 = x1*w1';
   M10_1 = M01_1';
   M11_1 = w1*w1';
   %
   % Making sure that M00_1 is invertible;
   % if not, drop this case
   %
   if abs(det(M00_1))>1.0e-008;
      H_1 = M11_1-M10_1*(M00_1\M01_1);
      c1 = (epsilon*w1')/M11_1;
      y1 = epsilon-(c1*w1);
      Sigma1 = (1/T)*y1*y1';
      if nf1==0;
         N01_1 = x1*z1';
         N10_1 = N01_1';
         N11_1 = z1*z1';
         G_1 = N11_1-N10_1*(M00_1\N01_1);
         if abs(det(G_1))<1.0e-008;
            nf1 = 1;
         end;
      end;
   else;
      nr1 = 0;
   end;
end;
if nr2>0;
   w2 = x2.*(ones(nr2,1)*trend);
   if df-(3*nr2)>10;
      w22 = x2.*(ones(nr2,1)*trend2);
      w23 = x2.*(ones(nr2,1)*trend3);
      z2 = [w2;w22;w23];
      checkz = min(abs(eig(z2*z2')));
      if checkz>1.0e-08;
         nf2 = 0;
      end;
      if nf2==0;
         b2 = (epsilon*z2')/(z2*z2');
         v2 = epsilon-(b2*z2);
         Omega2 = (1/T)*v2*v2';
      end;
   end;
   M00_2 = x2*x2';
   M01_2 = x2*w2';
   M10_2 = M01_2';
   M11_2 = w2*w2';
   %
   % Making sure that M00_2 is invertible;
   % if not, drop this case
   %
   if abs(det(M00_2))>1.0e-024;
      H_2 = M11_2-M10_2*(M00_2\M01_2);
      c2 = (epsilon*w2')/M11_2;
      y2 = epsilon-(c2*w2);
      Sigma2 = (1/T)*y2*y2';
      if nf2==0;
         N01_2 = x2*z2';
         N10_2 = N01_2';
         N11_2 = z2*z2';
         G_2 = N11_2-N10_2*(M00_2\N01_2);
         if abs(det(G_2))<1.0e-008;
            nf2 = 1;
         end;
      end;
   else;
      nr2 = 0;
   end;
end;
if nr3>0;
   w3 = x3.*(ones(nr3,1)*trend);
   if df-(3*nr3)>10;
      w32 = x3.*(ones(nr3,1)*trend2);
      w33 = x3.*(ones(nr3,1)*trend3);
      z3 = [w3;w32;w33];
      checkz = min(abs(eig(z3*z3')));
      if checkz>1.0e-08;
         nf3 = 0;
      end;
      if nf3==0;
         b3 = (epsilon*z3')/(z3*z3');
         v3 = epsilon-(b3*z3);
         Omega3 = (1/T)*v3*v3';
      end;
   end;
   M00_3 = x3*x3';
   M01_3 = x3*w3';
   M10_3 = M01_3';
   M11_3 = w3*w3';
   %
   % Making sure that M00_3 is invertible;
   % if not, drop this case
   %
   if abs(det(M00_3))>1.0e-045;
      H_3 = M11_3-M10_3*(M00_3\M01_3);
      c3 = (epsilon*w3')/M11_3;
      y3 = epsilon-(c3*w3);
      Sigma3 = (1/T)*y3*y3';
      if nf3==0;
         N01_3 = x3*z3';
         N10_3 = N01_3';
         N11_3 = z3*z3';
         G_3 = N11_3-N10_3*(M00_3\N01_3);
         if abs(det(G_3))<1.0e-008;
            nf3 = 1;
         end;
      end;
   else;
      nr3 = 0;
   end;
end;
if nr4>0;
   w4 = x4.*(ones(nr4,1)*trend);
   if df-(3*nr4)>10;
      w42 = x4.*(ones(nr4,1)*trend2);
      w43 = x4.*(ones(nr4,1)*trend3);
      z4 = [w4;w42;w43];
      checkz = min(abs(eig(z4*z4')));
      if checkz>1.0e-08;
         nf4 = 0;
      end;
      if nf4==0;
         b4 = (epsilon*z4')/(z4*z4');
         v4 = epsilon-(b4*z4);
         Omega4 = (1/T)*v4*v4';
      end;
   end;
   M00_4 = x4*x4';
   M01_4 = x4*w4';
   M10_4 = M01_4';
   M11_4 = w4*w4';
   %
   % Making sure that M00_4 is invertible;
   % if not, drop this case
   %
   if abs(det(M00_4))>1.0e-008;
      H_4 = M11_4-M10_4*(M00_4\M01_4);
      c4 = (epsilon*w4')/M11_4;
      y4 = epsilon-(c4*w4);
      Sigma4 = (1/T)*y4*y4';
      if nf4==0;
         N01_4 = x4*z4';
         N10_4 = N01_4';
         N11_4 = z4*z4';
         G_4 = N11_4-N10_4*(M00_4\N01_4);
         if abs(det(G_4))<1.0e-008;
            nf4 = 1;
         end;
      end;
   else;
      nr4 = 0;
   end;
end;
%
% Compute LM and F tests
%
for i=1:n;
   if nr1>0;
      epsi1 = w1*epsilon(i,:)';
      lmi = (1/Omega(i,i))*epsi1'*(H_1\epsi1);
      lmvec(i,1) = lmi;
      dfmat(i,1) = nr1;
      df1j = nr1;
      df2j = df-nr1;
      fj = (df2j/df1j)*((Omega(i,i)-Sigma1(i,i))/Sigma1(i,i));
      dfmat(i,2) = df1j;
      dfmat(i,3) = df2j;
      fvec(i,1) = fj;
      if nf1==0;
         epsi1j = z1*epsilon(i,:)';
         lmj = (1/Omega(i,i))*epsi1j'*(G_1\epsi1j);
         lmvec(i,2) = lmj;
         dfmat(i,4) = 3*nr1;
         df1i = 3*nr1;
         df2i = df-3*nr1;
         fi = (df2i/df1i)*((Omega(i,i)-Omega1(i,i))/Omega1(i,i));
         fvec(i,2) = fi;
         dfmat(i,5) = df1i;
         dfmat(i,6) = df2i;
      end;
   end;
   if nr2>0;
      epsi2 = w2*epsilon(i,:)';
      lmi = (1/Omega(i,i))*epsi2'*(H_2\epsi2);
      lmvec(i+n,1) = lmi;
      dfmat(i+n,1) = nr2;
      df1j = nr2;
      df2j = df-nr2;
      fj = (df2j/df1j)*((Omega(i,i)-Sigma2(i,i))/Sigma2(i,i));
      dfmat(i+n,2) = df1j;
      dfmat(i+n,3) = df2j;
      fvec(i+n,1) = fj;
      if nf2==0;
         epsi2j = z2*epsilon(i,:)';
         lmj = (1/Omega(i,i))*epsi2j'*(G_2\epsi2j);
         lmvec(i+n,2) = lmj;
         dfmat(i+n,4) = 3*nr2;
         df1i = 3*nr2;
         df2i = df-3*nr2;
         fi = (df2i/df1i)*((Omega(i,i)-Omega2(i,i))/Omega2(i,i));
         fvec(i+n,2) = fi;
         dfmat(i+n,5) = df1i;
         dfmat(i+n,6) = df2i;
      end;
   end;
   if nr3>0;
      epsi3 = w3*epsilon(i,:)';
      lmi = (1/Omega(i,i))*epsi3'*(H_3\epsi3);
      df1j = nr3;
      df2j = df-nr3;
      fj = (df2j/df1j)*((Omega(i,i)-Sigma3(i,i))/Sigma3(i,i));
      if nf3==0;
         df1i = 3*nr3;
         df2i = df-3*nr3;
         fi = (df2i/df1i)*((Omega(i,i)-Omega3(i,i))/Omega3(i,i));
         epsi3j = z3*epsilon(i,:)';
         lmj = (1/Omega(i,i))*epsi3j'*(G_3\epsi3j);
      end;
      if num==2;
         lmvec(i+n,1) = lmi;
         dfmat(i+n,1) = nr3;
         dfmat(i+n,2) = df1j;
         dfmat(i+n,3) = df2j;
         fvec(i+n,1) = fj;
         if nf3==0;
            lmvec(i+n,2) = lmj;
            dfmat(i+n,4) = 3*nr3;
            fvec(i+n,2) = fi;
            dfmat(i+n,5) = df1i;
            dfmat(i+n,6) = df2i;
         end;
      elseif num==3;
         if nr2>0;
            lmvec(i+2*n,1) = lmi;
            dfmat(i+2*n,1) = nr3;
            dfmat(i+2*n,2) = df1j;
            dfmat(i+2*n,3) = df2j;
            fvec(i+2*n,1) = fj;
            if nf3==0;
               lmvec(i+2*n,2) = lmj;
               dfmat(i+2*n,4) = 3*nr3;
               lmvec(i+2*n,2) = lmj;
               dfmat(i+2*n,4) = 3*nr3;
               fvec(i+2*n,2) = fi;
               dfmat(i+2*n,5) = df1i;
               dfmat(i+2*n,6) = df2i;
            end;
         else;
            lmvec(i+n,1) = lmi;
            dfmat(i+n,1) = nr3;
            dfmat(i+n,2) = df1j;
            dfmat(i+n,3) = df2j;
            fvec(i+n,1) = fj;
            if nf3==0;
               lmvec(i+n,2) = lmj;
               dfmat(i+n,4) = 3*nr3;
               fvec(i+n,2) = fi;
               dfmat(i+n,5) = df1i;
               dfmat(i+n,6) = df2i;
            end;
         end;
      else;
         lmvec(i+2*n,1) = lmi;
         dfmat(i+2*n,1) = nr3;
         dfmat(i+2*n,2) = df1j;
         dfmat(i+2*n,3) = df2j;
         fvec(i+2*n,1) = fj;
         if nf3==0;
            lmvec(i+2*n,2) = lmj;
            dfmat(i+2*n,4) = 3*nr3;
            fvec(i+2*n,2) = fi;
            dfmat(i+2*n,5) = df1i;
            dfmat(i+2*n,6) = df2i;
         end;
      end;
   end;
   if nr4>0;
      epsi4 = w4*epsilon(i,:)';
      lmi = (1/Omega(i,i))*epsi4'*(H_4\epsi4);
      df1j = nr4;
      df2j = df-nr4;
      fj = (df2j/df1j)*((Omega(i,i)-Sigma4(i,i))/Sigma4(i,i));
      if nf4==0;
         epsi4j = z4*epsilon(i,:)';
         lmj = (1/Omega(i,i))*epsi4j'*(G_4\epsi4j);
         df1i = 3*nr4;
         df2i = df-3*nr4;
         fi = (df2i/df1i)*((Omega(i,i)-Omega4(i,i))/Omega4(i,i));
      end;
      if num==2;
         lmvec(i+n,1) = lmi;
         dfmat(i+n,1) = nr4;
         dfmat(i+n,2) = df1j;
         dfmat(i+n,3) = df2j;
         fvec(i+n,1) = fj;
         if nf4==0;
            lmvec(i+n,2) = lmj;
            dfmat(i+n,4) = 3*nr4;
            fvec(i+n,2) = fi;
            dfmat(i+n,5) = df1i;
            dfmat(i+n,6) = df2i;
         end;
      elseif num==3;
         lmvec(i+2*n,1) = lmi;
         dfmat(i+2*n,1) = nr4;
         dfmat(i+2*n,2) = df1j;
         dfmat(i+2*n,3) = df2j;
         fvec(i+2*n,1) = fj;
         if nf4==0;
            lmvec(i+2*n,2) = lmj;
            dfmat(i+2*n,4) = 3*nr4;
            fvec(i+2*n,2) = fi;
            dfmat(i+2*n,5) = df1i;
            dfmat(i+2*n,6) = df2i;
         end;
      else;
         lmvec(i+3*n,1) = lmi;
         dfmat(i+3*n,1) = nr4;
         dfmat(i+3*n,2) = df1j;
         dfmat(i+3*n,3) = df2j;
         fvec(i+3*n,1) = fj;
         if nf4==0;
            lmvec(i+3*n,2) = lmj;
            dfmat(i+3*n,4) = 3*nr4;
            fvec(i+3*n,2) = fi;
            dfmat(i+3*n,5) = df1i;
            dfmat(i+3*n,6) = df2i;
         end;
      end;
   end;
end;

%
% Make sure there are degrees of freedom if test was not
% computed (to avoid NaN entries)
%

for i=1:size(dfmat,1);
   for j=1:6;
      if dfmat(i,j)==0;
         dfmat(i,j) = 1;
      end;
   end;
end;

pval = chis_cdf(lmvec(:,1),dfmat(:,1));
lmvec = [lmvec (1-pval)];
pval = chis_cdf(lmvec(:,2),dfmat(:,4));
lmvec = [lmvec (1-pval)];

pval = fdis_cdf(fvec(:,1),dfmat(:,2),dfmat(:,3));
fvec = [fvec (1-pval)];
pval = fdis_cdf(fvec(:,2),dfmat(:,5),dfmat(:,6));
fvec = [fvec (1-pval)];

nof = [nf1 nf2 nf3 nf4];
nrvec = [nr1 nr2 nr3 nr4];

%
% check for empty columns
%

delrow = [];
for i=1:(num*n);
   if lmvec(i,1:2)==zeros(1,2);
      delrow = [delrow;i];
   end;
end;
if isempty(delrow)==0;
   keeprow = [];
   for i=1:(num*n);
      done = 0;
      for j=1:size(delrow,1);
         if i==delrow(j,1);
            done = 1;
            j = size(delrow,1)+1;
         end;
      end;
      if done==0;
         keeprow = [keeprow;i];
      end;
   end;
   if size(keeprow,1)<(num*n);
      if size(keeprow,1)>0;
         lmvec = lmvec(keeprow,:);
         fvec = fvec(keeprow,:);
         dfmat = dfmat(keeprow,:);
      else;
         lmvec = [];
         fvec = [];
         dfmat = [];
         nof = ones(1,4);
         nrvec = zeros(1,4);
      end;
   end;
end;

%
% End of function lstr1.m
%
