%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                                                                %
%          MatLab GUI dialog for displaying graphs of            %
%          marginal parameter distributions                      %
%                                                                %
%          Function: marginal_graphdlg                           %
%                                                                %
%          Input:                                                %
%                       selector: case switch function           %
%                       cm: structure with model info            %
%                       param_type: string with parameter group  %
%                          name                                  %
%                       param_str: string matrix with parameter  %
%                          names                                 %
%                       posterior_data: structure with draws     %
%                          from the posterior distribution       %
%                       graph_title: string vector with base     %
%                          title                                 %
%                       graph_type: string vector being function %
%                          or histogram depending on the type    %
%                          of graph that should be plotted       %
%                       restricted: boolean variable for         %
%                          restricted beta                       %
%                       h_vec: needed for beta                   %
%                       H_mat: needed for beta                   %
%                                                                %
%          Output:                                               %
%          	            h: handle                                %
%                                                                %
%          This version:        June 16, 2006                    %
%          Written by:          Anders Warne                     %
%                               Copyright (c) 2001-2006          %
%                                                                %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function h = marginal_graphdlg(selector,cm,param_type,param_str,posterior_data,graph_type,restricted,h_vec,H_mat)

if strcmp(selector,'init')==1;
   thisexit = 0;
   curr_ini = find_inisettings;
   if isnumeric(curr_ini.external)==0;
      curr_ini.external = 0;
   end;
   cm.ini = curr_ini;
   cm.posterior_data = posterior_data;
   cm.realscrsz = get(0,'ScreenSize');
   cm.graph_type = graph_type;
   cm.param_type = lower(param_type);
   if strcmp(cm.param_type,'beta')==1;
      add_height = 35;
   else;
      add_height = 0;
   end;
   cm.comp_mean = 0;
   if restricted==0;
      cm.prior_inf_alpha = 1;
      cm.prior_inf_omega = 1;
   end;
   cm.h_vec = h_vec;
   cm.H_mat = H_mat;
   if size(H_mat,2)<(size(cm.beta,1)-cm.r)*cm.r;
      cm.compute_beta_mean = 1;
   else;
      cm.compute_beta_mean = 0;
   end;
else
   thisexit = 1;
   param_graphgui = findobj('Type','figure','Tag','MarginParamGraphDlg');
   param_graph = get(param_graphgui,'UserData');
end;

switch selector;
   case 'init';
      param_graphgui = figure('Color',get(0,'defaultuicontrolbackgroundcolor'), ...
         'FileName','marginal_graphdlg.m', ...
         'MenuBar','none', ...
         'PaperUnits','points', ...
         'Units','pixels', ...
         'Position',[(cm.realscrsz(3)-341)/2 (cm.realscrsz(4)-140-add_height)/2 341 140+add_height], ...
         'Tag','MarginParamGraphDlg', ...
         'Visible','off', ...
         'Resize','off', ...
         'Name',['Marginal Distribution of ' param_type], ...
         'NumberTitle','off', ...
         'CloseRequestFcn','marginal_graphdlg done', ...
         'ToolBar','none');

      if matlabnumber>=7.0;
         set(param_graphgui,'DockControl','off');
      end;

      %
      % Outer axis
      %

      axesbox([2 2 339 130+add_height],'Graphics',45,[0.5 1]);

      imageaxes = axes('Units','pixels', ...
         'Box','on', ...
         'TickLength',[0 0], ...
         'CameraUpVector',[0 1 0], ...
         'Color',get(0,'defaultuicontrolbackgroundcolor'), ...
         'Position',[24 80+add_height 39 39], ...
         'XTickLabelMode','manual', ...
         'YTickLabelMode','manual', ...
         'Tag','AxisLogo', ...
         'XColor',get(0,'defaultuicontrolbackgroundcolor'), ...
         'YColor',get(0,'defaultuicontrolbackgroundcolor'), ...
         'ZColor',get(0,'defaultuicontrolbackgroundcolor'));

      mycons = getstoredimages;
      if curr_ini.use_toolbar_bgcolor==1;
         bgcolor = (1/255)*(round(255*get(0,'defaultuicontrolbackgroundcolor'))-[8 4 8]);
         bgcolor(1) = max(0,bgcolor(1));
         bgcolor(2) = max(0,bgcolor(2));
         bgcolor(3) = max(0,bgcolor(3));
      else;
         bgcolor = get(0,'defaultuicontrolbackgroundcolor');
      end;
      %
      image('CData',mirrorimage(maketrans(mycons.betamatlab_image,[160 160 164],bgcolor)),'Clipping','off','Parent',imageaxes);

      param_graph.icon = uicontrol('Units','pixels', ...
         'BackgroundColor',get(0,'defaultuicontrolbackgroundcolor'), ...
         'Position',[24 50+add_height 200 20], ...
         'FontSize',curr_ini.gui_fontsize, ...
         'FontName',curr_ini.gui_fontname, ...
         'FontWeight',curr_ini.gui_fontweight, ...
         'FontAngle',curr_ini.gui_fontangle, ...
         'Style','text', ...
         'String',['Posterior for ' lower(param_type) ' with ' num2str(cm.save_draws,'%0.0f') ' draws'], ...
         'HorizontalAlignment','left', ...
         'Tag','SampleText');

      %
      % Select variable
      %

      param_graph.variables = uicontrol('Units','pixels', ...
         'BackgroundColor',get(0,'defaultuicontrolbackgroundcolor'), ...
         'Position',[24 15+add_height 80 20], ...
         'FontSize',curr_ini.gui_fontsize, ...
         'FontName',curr_ini.gui_fontname, ...
         'FontWeight',curr_ini.gui_fontweight, ...
         'FontAngle',curr_ini.gui_fontangle, ...
         'Style','text', ...
         'UserData',cm, ...
         'String','Parameter:', ...
         'HorizontalAlignment','left', ...
         'Tag','VariableText');


      param_graph.popup = uicontrol('Units','pixels', ...
         'BackgroundColor',[1 1 1], ...
         'Position',[105 20+add_height 100 20], ...
         'Style','popupmenu', ...
         'FontSize',curr_ini.gui_fontsize, ...
         'FontName',curr_ini.gui_fontname, ...
         'FontWeight',curr_ini.gui_fontweight, ...
         'FontAngle',curr_ini.gui_fontangle, ...
         'String',param_str, ...
         'HorizontalAlignment','center', ...
         'Tag','GraphPopup',...
         'Value',1);

      if strcmp(cm.param_type,'beta')==1;
         param_graph.change_sign = uicontrol('Units','pixels', ...
               'BackgroundColor',get(0,'defaultuicontrolbackgroundcolor'), ...
               'Position',[24 16 170 20], ...
               'FontSize',curr_ini.gui_fontsize, ...
               'FontName',curr_ini.gui_fontname, ...
               'FontWeight',curr_ini.gui_fontweight, ...
               'FontAngle',curr_ini.gui_fontangle, ...
               'Style','checkbox', ...
               'Value',0, ...
               'Enable','on', ...
               'Visible','on', ...
               'HorizontalAlignment','left', ...
               'String','Change sign of parameter', ...
               'Tag','CheckChangeSign');
      end;

      %
      % Create graph button
      %

      param_graph.show = uicontrol('Units','pixels', ...
         'BackgroundColor',get(0,'defaultuicontrolbackgroundcolor'), ...
         'Position',[236 55 90 20], ...
         'String','Display', ...
         'FontSize',curr_ini.gui_fontsize, ...
         'FontName',curr_ini.gui_fontname, ...
         'FontWeight',curr_ini.gui_fontweight, ...
         'FontAngle',curr_ini.gui_fontangle, ...
         'CallBack','marginal_graphdlg showgraph', ...
         'Enable','on', ...
         'UserData','working', ...
         'Tag','Done');

      %
      %  Done button
      %

      param_graph.done = uicontrol('Units','pixels', ...
         'BackgroundColor',get(0,'defaultuicontrolbackgroundcolor'), ...
         'Position',[236 20 90 20], ...
         'String','Done', ...
         'FontSize',curr_ini.gui_fontsize, ...
         'FontName',curr_ini.gui_fontname, ...
         'FontWeight',curr_ini.gui_fontweight, ...
         'FontAngle',curr_ini.gui_fontangle, ...
         'CallBack','marginal_graphdlg done', ...
         'Enable','on', ...
         'UserData','waiting', ...
         'Tag','Done');

      set(param_graphgui,'UserData',param_graph,...
         'HandleVisibility','callback');
   case 'showgraph'
      set(param_graphgui,'Visible','off');
      drawnow;
      %
      if isfield(param_graph,'change_sign')==1;
         if get(param_graph.change_sign,'Value')==0;
            mult_parm = 1;
         else;
            mult_parm = -1;
         end;
      else;
         mult_parm = 1;
      end;
      %
      cm = get(param_graph.variables,'UserData');
      curr_ini = cm.ini;
      curr_val = get(param_graph.popup,'Value');
      curr_param = get(param_graph.popup,'String');
      if curr_val==size(curr_param,1);
         new_val = 1;
      else;
         new_val = curr_val+1;
      end;
      curr_param = strtrim(curr_param(curr_val,:));
      set(param_graph.popup,'Value',new_val);
      i1 = findstr(curr_param,'(');
      if isempty(i1)==0;
         tag_name = curr_param(1,1:i1-1);
         i2 = findstr(curr_param,')');
         index_str = curr_param(1,i1+1:i2-1);
         index_str = strrep(index_str,',','-');
         v_istr = vecstr(strrep(index_str,'-',' '));
      else;
         tag_name = curr_param;
         v_istr = '';
      end;
      tex_param = ['\' tag_name];
      if isempty(v_istr)==0;
         tex_param = [tex_param  '_{' strtrim(v_istr(1,:)) strtrim(v_istr(2,:)) '}'];
         tag_name = [tag_name '-' index_str '-plot-'];
      else;
         tag_name = [tag_name '-plot-'];
      end;
      lag_param = '';
      if size(v_istr,1)>2;
         lag_str = [' for lag ' strtrim(v_istr(3,:))];
      end;
      %
      % create data matrices
      %
      if isempty(cm.Z2)==0;
         [D_mu,D_sigma,G_mu,G_sigma] = prior_short_coint(cm);
      else;
         D_mu = [];
         D_sigma = [];
         G_sigma = [];
         G_mu = [];
      end;
      if cm.amat_type==1;
         A = cm.rank(cm.n+1).Omega;
         q = cm.n+2;
      else;
         A = cm.lambda_a*eye(cm.n);
         q = cm.q_parm;
      end;
      Z0 = cm.Z0;
      Z1 = cm.Z1;
      if cm.d>0;
         D = cm.Z2(1:cm.d,:);
      else;
         D = [];
      end;
      if (isempty(cm.Z2)==0)&(size(cm.Z2,1)>cm.d);
         Z2 = cm.Z2(1+cm.d:size(cm.Z2,1),:);
      else;
         Z2 = [];
      end;
      %
      % The integration matrix N
      %
      if (cm.prior_inf_shortrun==2)&(cm.prior_inf_deter==2);
         %
         % improper priors on delta and Gamma
         %
         if isempty(cm.Z2)==0;
            Z = cm.Z2;
            N = eye(cm.T)-Z'*inverse(Z*Z')*Z;
         else;
            N = eye(cm.T);
         end;
      elseif (cm.prior_inf_shortrun==1)&(cm.prior_inf_deter==2);
         %
         % improper prior on delta
         %
         if isempty(cm.Z2)==0;
            if cm.d>0;
               M_D = eye(cm.T)-(D'*inverse(D*D')*D);
            else;
               M_D = eye(cm.T);
            end;
            if size(cm.Z2,1)>cm.d;
               N = M_D-(M_D*Z2'*inverse(((Z2*M_D*Z2')+inverse(G_sigma)))*Z2*M_D);
            else;
               N = M_D;
            end;
         else;
            N = eye(cm.T);
         end;
      elseif (cm.prior_inf_shortrun==2)&(cm.prior_inf_deter==1);
         %
         % improper prior on Gamma
         %
         if isempty(cm.Z2)==0;
            if cm.d>0;
               if size(cm.Z2)>cm.d;
                  M_Z2 = eye(cm.T)-(Z2'*inverse(Z2*Z2')*Z2);
                  N = M_Z2-(M_Z2*D'*inverse(((D*M_Z2*D')+inverse(D_sigma)))*D*M_Z2);
               else;
                  N = eye(cm.T)-D'*inverse(((D*D')+inverse(D_sigma)))*D;
               end;
            else;
               M_Z2 = eye(cm.T)-(Z2'*inverse(Z2*Z2')*Z2);
               N = M_Z2;
            end;
         else;
            N = eye(cm.T);
         end;
      else;
         %
         % proper priors on delta and Gamma
         %
         if isempty(cm.Z2)==0;
            Z = cm.Z2;
            if cm.d>0;
               if size(cm.Z2)>cm.d;
                  F_sigma = [[D_sigma zeros(cm.d,size(G_sigma,1))];[zeros(size(G_sigma,1),cm.d) G_sigma]];
               else;
                  F_sigma = D_sigma;
               end;
            else;
               F_sigma = G_sigma;
            end;
            N = eye(cm.T)-(Z'*inverse(((Z*Z')+inverse(F_sigma)))*Z);
         else;
            N = eye(cm.T);
         end;
      end;
      C_0 = Z0*N*Z1';
      C_1 = Z1*N*Z1';
      if cm.prior_inf_alpha==1;
         C_1 = C_1+(1/cm.lambda_alpha^2)*eye(cm.n);
      end;
      %
      % we now create the marginal density
      %
      if strcmp(cm.param_type,'beta')==1;
         %
         % we're dealing with the cointegration parameters.
         %
         % step 1: set up a grid
         %
         row_col = strrep(curr_param,'beta','');
         row_col = strrep(row_col,'(','');
         row_col = strrep(row_col,')','');
         row_col = mystr2num(strtrim(strrep(row_col,',',' ')));
         row = row_col(1,2);
         col = row_col(1,1);
         %
         % select the proper row vector of H_mat for our parameter
         %
         h_elem = row+((col-1)*size(cm.posterior_data(1).beta_n,1));
         %
         % collect the draws to find the maximum and the minimum
         %
         param_vec = zeros(cm.save_draws,1);
         for i=1:cm.save_draws;
            param_vec(i,1) = cm.posterior_data(i).beta_n(row,col);
         end;
         p_min = min(param_vec);
         p_max = max(param_vec);
         p_diff = p_max-p_min;
         p_min = p_min-(0.5*p_diff);
         p_max = p_max+(0.5*p_diff);
         p_change = (p_max-p_min)/400;
         %
         % input for the progress dialog
         %
         input_var.title = ['Marginalizing ' tex_param];
         input_var.facecolor = curr_ini.progress_facecolor;
         input_var.startfacecolor = curr_ini.progress_startfacecolor;
         input_var.edgecolor = curr_ini.progress_edgecolor;
         input_var.bgcolor = curr_ini.progress_bgcolor;
         input_var.label = [tex_param ' ='];
         input_var.stop = 0;
         input_var.clock = 1;
         input_var.name = ['Marginalizing ' curr_param];
         mywait = progressdlg(0,input_var);
         set(mywait,'Color',get(0,'defaultuicontrolbackgroundcolor'));
         drawnow;
         abort = '';
         %
         p_value = p_min;
         num = 0;
         %
         % parm_matrix contains the beta parameter value and the estimated density value
         %
         parm_matrix = zeros(400,2);
         while num<400;
            num = num+1;
            %
            % update the progress dialog
            %
            abort = get(mywait,'UserData');
            if (strcmp(abort,'cancel')==1);
               break;
            else;
               progressdlg([(p_value-p_min)/(p_max-p_min) mult_parm*p_value]);
            end;
            %
            % compute the density value at the current point
            %
            d_value = 0;
            for i=1:cm.save_draws;
               %
               % determine the mean and the variance of of psi_i
               %
               alpha = cm.posterior_data(i).alpha_n;
               Omega = cm.posterior_data(i).Omega_n;
               %
               cov_mat = kron(alpha'*inverse(Omega)*alpha,C_1);
               mean_vec = vec(C_0'*inverse(Omega)*alpha);
               %
               cov_mat_b_inv = cov_mat;
               cov_mat = inverse(cov_mat);
               mean_vec = cov_mat*mean_vec;
               %
               % now we transform this mean and covariance into psi rather than vec(beta)
               %
               cov_mat_psi = inverse((cm.H_mat'*cov_mat_b_inv*cm.H_mat));
               mean_vec = cov_mat_psi*cm.H_mat'*cov_mat_b_inv*(mean_vec-cm.h_vec);
               %
               % select the parameter mean and variance of interest
               %
               psi_mean = cm.H_mat(h_elem,:)*mean_vec;
               psi_var = cm.H_mat(h_elem,:)*cov_mat_psi*cm.H_mat(h_elem,:)';
               %
               % now we compute the density value at the current parameter point
               %
               e_value = -((p_value-psi_mean)^2)/(2*psi_var);
               if exp(e_value)>0;
                  d_value = d_value+((1/cm.save_draws)*(1/sqrt(psi_var))*exp(e_value));
               end;
            end;
            parm_matrix(num,1) = p_value;
            parm_matrix(num,2) = d_value;
            p_value = p_value+p_change;
         end;
         %
         % close the progress dialog
         %
         set(mywait,'UserData','done');
         close(mywait);
         drawnow;
         %
         %  compute statistics from the marginal distribution
         %
         if strcmp(abort,'cancel')==0;
            if get(param_graph.change_sign,'Value')==1;
               %
               % change sign of beta parameter and make sure that everything is flipped
               %
               parm_matrix = [reverse(-1*parm_matrix(:,1)) reverse(parm_matrix(:,2))];
            end;
            if strcmp(cm.graph_type,'function')==1;
               [parm_matrix,p_mode,p_median,p_lower,p_upper,p_mean] = parameter_stats(parm_matrix,cm.compute_beta_mean,cm.cumdist,cm.cbs);
            else;
               parms = parameter_hist_fcn(parm_matrix);
            end;
         end;
      elseif strcmp(cm.param_type,'alpha')==1;
         %
         % step 1: set up a grid
         %
         row_col = strrep(curr_param,'alpha','');
         row_col = strrep(row_col,'(','');
         row_col = strrep(row_col,')','');
         row_col = mystr2num(strtrim(strrep(row_col,',',' ')));
         row = row_col(1,1);
         col = row_col(1,2);
         a_elem = row+(cm.n*(col-1));
         %
         % collect the draws to find the maximum and the minimum
         %
         param_vec = zeros(cm.save_draws,1);
         for i=1:cm.save_draws;
            param_vec(i,1) = cm.posterior_data(i).alpha_n(row,col);
         end;
         p_min = min(param_vec);
         p_max = max(param_vec);
         p_diff = p_max-p_min;
         p_min = p_min-(0.5*p_diff);
         p_max = p_max+(0.5*p_diff);
         p_change = (p_max-p_min)/400;
         %
         % input for the progress dialog
         %
         input_var.title = ['Marginalizing ' tex_param];
         input_var.facecolor = curr_ini.progress_facecolor;
         input_var.startfacecolor = curr_ini.progress_startfacecolor;
         input_var.edgecolor = curr_ini.progress_edgecolor;
         input_var.bgcolor = curr_ini.progress_bgcolor;
         input_var.label = [tex_param ' ='];
         input_var.stop = 0;
         input_var.clock = 1;
         input_var.name = ['Marginalizing ' curr_param];
         mywait = progressdlg(0,input_var);
         set(mywait,'Color',get(0,'defaultuicontrolbackgroundcolor'));
         drawnow;
         abort = '';
         %
         p_value = p_min;
         num = 0;
         %
         % parm_matrix contains the beta parameter value and the estimated density value
         %
         parm_matrix = zeros(400,2);
         while num<400;
            num = num+1;
            %
            % update the progress dialog
            %
            abort = get(mywait,'UserData');
            if (strcmp(abort,'cancel')==1);
               break;
            else;
               progressdlg([(p_value-p_min)/(p_max-p_min) mult_parm*p_value]);
            end;
            %
            % compute the density value at the current point
            %
            d_value = 0;
            i=1;
            for i=1:cm.save_draws;
               %
               % determine the mean and the variance of of psi_i
               %
               beta = cm.posterior_data(i).beta_n;
               Omega = cm.posterior_data(i).Omega_n;
               %
               cov_mat = inverse(beta'*C_1*beta);
               alpha_hat = C_0*beta*cov_mat;
               cov_mat = kron(cov_mat,Omega);
               %
               % select the parameter mean and variance of interest
               %
               alpha_h = alpha_hat(row,col);
               alpha_h_var = cov_mat(a_elem,a_elem);
               %
               % now we compute the density value at the current parameter point
               %
               e_value = -((p_value-alpha_h)^2)/(2*alpha_h_var);
               if exp(e_value)>0;
                  d_value = d_value+((1/cm.save_draws)*(1/sqrt(alpha_h_var))*exp(e_value));
               end;
            end;
            parm_matrix(num,1) = p_value;
            parm_matrix(num,2) = d_value;
            p_value = p_value+p_change;
         end;
         %
         % close the progress dialog
         %
         set(mywait,'UserData','done');
         close(mywait);
         drawnow;
         %
         %  compute statistics from the marginal distribution
         %
         if strcmp(abort,'cancel')==0;
            if strcmp(cm.graph_type,'function')==1;
               [parm_matrix,p_mode,p_median,p_lower,p_upper,p_mean] = parameter_stats(parm_matrix,1,cm.cumdist,cm.cbs);
            else;
               parms = parameter_hist_fcn(parm_matrix);
            end;
         end;
      end;
      %
      %
      set(param_graphgui,'Visible','on');
      drawnow;
      drawnow;
      %
      %
      if strcmp(abort,'cancel')==0;
         %
         % prepare for the graph
         %
         if curr_ini.colors==1;
            col.vec_1 = [0 0 1];
            col.vec_2 = [1 0 0];
            col.vec_3 = [0 0.75 0];
            col.vec_4 = [1 0.5 0];
            col.vec_5 = [0 0.75 0.75];
            col.vec_6 = [0 0 0];
            col.vec_7 = (1/255)*[255 28 28];
            col.vec_8 = [0 1 0];
            col.vec_9 = (1/255)*[255 255 177];
         else;
            col.vec_1 = [0 0 0];
            col.vec_2 = [0.7 0.7 0.7];
            col.vec_3 = [0.6 0.6 0.6];
            col.vec_4 = [0.5 0.5 0.5];
            col.vec_5 = [0.4 0.4 0.4];
            col.vec_6 = [0.3 0.3 0.3];
            col.vec_7 = [0.8 0.8 0.8];
            col.vec_8 = [0.2 0.2 0.2];
            col.vec_9 = col.vec_3;
         end;
         %
         p_plot = figure('Units','pixels', ...
            'Position',[(cm.realscrsz(3)-512)/2 (cm.realscrsz(4)-384)/2 512 384], ...
            'Menubar','none', ...
            'Toolbar','none', ...
            'NumberTitle','off', ...
            'Visible','off', ...
            'Name',['Marginal Distribution of ' curr_param]);
         %
         % default figure properties
         %
         setfigureproperties(p_plot);
         %
         if strcmp(cm.graph_type,'function')==1;
            dist_handle = plot(parm_matrix(:,1),parm_matrix(:,2),'-','LineWidth',1,'Color',col.vec_1,'Tag',tag_name);
            hold('on');
            %
            y_lim = get(gca,'YLim');
            x_lim = get(gca,'XLim');
            y_lim = [0 1.1*y_lim(1,2)];
            set(gca,'YLim',y_lim);
            %
            % add mean, mode, median
            %
            if isempty(p_mean)==0;
               new_line = plot(p_mean*ones(1,2),y_lim,'Color',col.vec_2,'LineStyle','--','LineWidth',1);
               dist_handle = [dist_handle new_line];
               hold('on');
            end;
            new_line = plot(p_mode*ones(1,2),y_lim,'Color',col.vec_5,'LineStyle','-','LineWidth',1);
            dist_handle = [dist_handle new_line];
            hold('on');
            new_line = plot(p_median*ones(1,2),y_lim,'Color',col.vec_3,'LineStyle','-.','LineWidth',1);
            dist_handle = [dist_handle new_line];
            %
            % plotting edge line for confidence band
            %
            if isempty(p_lower)==0;
               hold('on');
               plot(p_lower*ones(1,2),y_lim,'Color',col.vec_6,'LineStyle',':','LineWidth',1,'Tag','nolegend-lower');
               hold('on');
               plot(p_upper*ones(1,2),y_lim,'Color',col.vec_6,'LineStyle',':','LineWidth',1,'Tag','nolegend-upper');
               add_conf_str = [' and with ' num2str(cm.cbs) '% confidence bands'];
            else;
               add_conf_str = '';
            end;
            %
            % check if we should add a vertical zero line
            %
            if (x_lim(1,1)<0)&(x_lim(1,2)>0);
               hold('on');
               plot(zeros(2,1),y_lim','-','LineWidth',1,'Color','k');
            end;
            %
            if cm.cumdist==0;
               setaxesfonts('Title',['Marginal posterior density for ' tex_param ' based on ' num2str(cm.save_draws,'%0.0f') ' draws' add_conf_str],'XLabel','Value','YLabel','Density');
            else;
               setaxesfonts('Title',['Marginal posterior distribution for ' tex_param ' based on ' num2str(cm.save_draws,'%0.0f') ' draws' add_conf_str],'XLabel','Value','YLabel','Frequency');
            end;
            leg_pos = 1;
            if cm.cumdist==1;
               leg_pos = 4;
            end;
            if isempty(p_mean)==0;
               myleg = legend_fcn(dist_handle,curr_param,'Mean','Mode','Median',leg_pos);
            else;
               myleg = legend_fcn(dist_handle,curr_param,'Mode','Median',leg_pos);
            end;
            setaxesfonts(myleg);
            set(gca,'UserData',myleg);
            toolbar_fcn(gca);
         else;
            [n1,n2] = hist(parms,cm.numcells);
            n1 = n1/size(parms,1);
            if cm.cumdist==1;
               n1 = cumsum(n1);
            end;
            %
            n3 = bar(n2,n1,0.75);
            this_axes = gca;
            if curr_ini.colors==1;
               set(n3,'FaceColor',[0 0 0.8],'EdgeColor','k','LineWidth',1,'Tag','autopatch');
            else;
               set(n3,'FaceColor',[0.5 0.5 0.5],'EdgeColor','k','LineWidth',1,'Tag','autopatch');
            end;
            if cm.cumdist==1;
               set(gca,'YLim',[0 1.1]);
            end;
            setaxesfonts('Title',['Marginal posterior distribution for ' tex_param ' based on ' num2str(cm.save_draws,'%0.0f') ' draws'],'XLabel','Value','YLabel','Frequency');
            toolbar_fcn(gca);
         end;
         %
         if curr_ini.external==0;
            set(p_plot,'Visible','on');
         else;
            file = [cm.outputdir cm.modelfile '-' tag_name cm.graph_type];
            set(0,'CurrentFigure',p_plot);
            stat = autosavefigure(p_plot,file);
            if isempty(stat)==0;
               if stat==0;
                  autoshowfigure(file);
               end;
            end;
         end;
      end;
   case 'done'
      set(param_graph.done,'UserData','done');
      set(param_graphgui,'UserData',param_graph);
end;

if thisexit==0;
   if (iswinnt==1)&(iscompiled==1)&(curr_ini.loadguiicon==1);
      prog = ['bin\runwindow.exe ' fix_path([pwd '\bin\seticon.exe']) ' ' get(param_graphgui,'Name') ' ' fix_filepath([pwd '\icons\matfile.ico'])];
      dos(prog);
      if (strcmp(curr_ini.pstoolsdir,'')==0)&(appexist([curr_ini.pstoolsdir 'pslist.exe'])==1);
         %
         % now we need to determine the pid
         %
         pid = getpid('seticon');
         set(param_graph.icon,'UserData',pid);
      end;
   end;
   %
   set(param_graphgui,'Visible','on');
   drawnow;
   if (iswinnt==1)&(iscompiled==1)&(matlabnumber<7);
      drawnow;
      prog = ['bin\run_prog.exe ' fix_path([pwd '\bin\setfocus.exe']) ' ' get(param_graphgui,'Name') ' 1'];
      dos(prog);
      drawnow;
   elseif matlabnumber>=7;
      uicontrol(param_graph.done);
   end;
   waitfor(param_graph.done,'UserData','done');
end;

if nargout==1;
   h = param_graphgui;
end;

%
% end of marginal_graphdlg.m
%
