%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                                                                %
%          MatLab function for estimating nxn matrix B_0 such    %
%          that B_0*B_0' = Omega, where vec(B_0) = S*gamma or    %
%          R*vec(B_0) = 0. Estimation is conditional on C.       %
%                                                                %
%          Function: vlaar                                       %
%                                                                %
%          Input:                                                %
%                       tm: structure with model info            %
%                       type_var: string determining type of     %
%                           restrictions to use                  %
%                       Psi: parameters on short-run dynamics    %
%                           and deterministics                   %
%                       alpha: parameters on levels relations    %
%                       beta: cointegration space                %
%                       epsilon: residuals                       %
%                       Z1: levels variables                     %
%                       Z2: changes and deterministics           %
%                       B_init: initial values for B_0           %
%                       step: parameter to multiply with         %
%                           parameter change in algorithm        %
%                       waitbar: 0-1 indicator whether to        %
%                           display a progress dialog or not     %
%                                                                %
%          Output:                                               %
%                        status: indicator of success            %
%                           (0 OK, 1 failure, 2 bad initial      %
%                            B matrix)                           %
%                        abort: string indicating if estimation  %
%                           was aborted or not                   %
%          	             B_0: nxn matrix with structural         %
%                           coefficients                         %
%                        S: n^2xq matrix mapping free parameters %
%                           gamma into vec(B_0)                  %
%                        gamma: the estimated free parameters    %
%                           of B_0                               %
%                        Sigma_B0: covariance matrix for B_0     %
%                        infomat: 1x2 vector with number of      %
%                           iterations and value of convergence  %
%                           criterion                            %
%                        R_l: the long-run restrictions          %
%                        Rmat: the restrictions matrix           %
%                                                                %
%          This version:        June 16, 2006                    %
%          Written by: 	        Anders Warne                     %
%                               Copyright (c) 2001-2006          %
%                                                                %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function [status,abort,B_0,S,gamma,Sigma_B0,infomat,R_l,Rmat,rank_test] = vlaar(tm,type_var,Psi,alpha,beta,epsilon,Z1,Z2,B_init,step,waitbar)

if nargin<11;
   waitbar = 0;
end;
if nargin<10;
   step = 1;
end;

r = size(alpha,2);
[n T] = size(epsilon);
Omega = (1/T)*epsilon*epsilon';

local_status = 0;
status = 1;
abort = '';
B_0 = [];
S = [];
gamma = [];
Sigma_B0 = [];
infomat = [];
R_l = [];
Rmat = [];
rank_test = [];

%
% check the ini file
%

curr_ini = find_inisettings;

if curr_ini.convcrit<10;
   ch_convcrit = ['1.0e-00' num2str(curr_ini.convcrit)];
else;
   ch_convcrit = ['1.0e-0' num2str(curr_ini.convcrit)];
end;
ccrit = mystr2num(ch_convcrit);

%
% checkin the type of restrictions we have
%

if strcmp(type_var,'short_run')==1;
   file = strcat(tm.modelsdir,'F-',int2str(n),'x',int2str(r),'-',tm.modelfile,'.rest');
   F = loaddata(file);
   [Rmat,rvec] = rmatrix(F,n,n);
   R_l = [];
elseif strcmp(type_var,'long_run')==1;
   file = strcat(tm.modelsdir,'LF-',int2str(n),'x',int2str(r),'-',tm.modelfile,'.rest');
   LF = loaddata(file);
   [Rmat,rvec] = rmatrix(LF,n,n);
   R_l = Rmat;
   %
   % need to compute a C matrix also for this case, i.e., r==n
   %
   if r<n;
      C = cmat(Psi,alpha,beta,epsilon,Z1,Z2,tm.p,tm.d,0,[]);
   else;
      %
      % now r=n
      %
      C = cmat_stat(Psi,alpha,beta,epsilon,Z1,Z2,tm.p,tm.d,0,[]);
   end;
   Rmat = Rmat*kron(eye(n),C);
else;
   file = strcat(tm.modelsdir,'F-',int2str(n),'x',int2str(r),'-',tm.modelfile,'.rest');
   F = loaddata(file);
   [Rmat_sr,rvec_sr] = rmatrix(F,n,n);
   file = strcat(tm.modelsdir,'LF-',int2str(n),'x',int2str(r),'-',tm.modelfile,'.rest');
   LF = loaddata(file);
   [Rmat_lr,rvec_lr] = rmatrix(LF,n,n);
   R_l = [zeros(size(Rmat_sr));Rmat_lr];
   if r<n;
      C = cmat(Psi,alpha,beta,epsilon,Z1,Z2,tm.p,tm.d,0);
      Rmat_lr = Rmat_lr*kron(eye(n),C);
   else;
      C = cmat_stat(Psi,alpha,beta,epsilon,Z1,Z2,tm.p,tm.d,0,[]);
      Rmat_lr = Rmat_lr*kron(eye(n),C);
   end;
   Rmat = [Rmat_sr;Rmat_lr];
   rvec = [rvec_sr;rvec_lr];
end;

S = null(Rmat,'r');
d = (Rmat'/(Rmat*Rmat'))*rvec;

%
% Initialize B_0 through an eigenvalue decomposition of Omega
% and map into gamma via S. This mapping is the solution to the
% minimization problem:
%
%  min F(gamma) = (vec(B)-S*gamma-d)'*(vec(B)-S*gamma-d)
%
% Hence we pick B first such that for Omega = U*V*U', where V
% is diagonal with the eigenvalues of Omega and U are the eigenvectors
% B = U*sqrt(V). Next, the solution to F is given by gamma = inverse(S*S')*S*(vec(B)-d).
% Finally, we let the initial value of B be given by:
% vec(B) = S*gamma+d. This means that the initial value of B satisfies
% the restrictions, should have full rank, and is close to satisfying
% B*B' = Omega
%

if isempty(B_init)==1;
   [U,V] = eig(Omega);
   B_init = U*sqrt(V);
else;
   %
   % check to make sure B_init is non-singular
   %
   if rank(B_init)~=n;
      [U,V] = eig(Omega);
      B_init = U*sqrt(V);
      local_status = 2;
   end;
end;

gamma = inverse(S'*S)*(S'*(vec(B_init)-d));

B_0 = vecinv(S*gamma+d,n);
if rank(B_0)<n;
   return;
end;
B_0_inv = B_0\eye(n);
grad = -T*S'*kron(B_0_inv,B_0_inv')*(vec(B_0')-( kron(Omega,eye(n))*vec(B_0_inv) ) );
max_grad = max(abs(grad));
%
% setting up a progress dialog if needed and asked for
%
showwaitbar = 0;
if (waitbar==1)&(max_grad>ccrit);
   input_var.title = ['Progress for maximum of ' int2str(curr_ini.maxit) ' iterations'];
   input_var.facecolor = curr_ini.progress_facecolor;
   input_var.startfacecolor = curr_ini.progress_startfacecolor;
   input_var.edgecolor = curr_ini.progress_edgecolor;
   input_var.bgcolor = curr_ini.progress_bgcolor;
   input_var.stop = 1;
   input_var.clock = 0;
   input_var.name = 'ML Estimation of B';
   mywait = progressdlg(0,input_var);
%   set(mywait,'Name','ML Estimation of B');
   set(mywait,'Color',get(0,'defaultuicontrolbackgroundcolor'));
   drawnow;
   showwaitbar = 1;
end;
Nn = ncommutation(n);
it = 0;
while ((it<=curr_ini.maxit)&(max_grad>ccrit));
   it = it+1;
   if waitbar==1;
      abort = get(mywait,'UserData');
      if (strcmp(abort,'cancel')==1)|(strcmp(abort,'done')==1);
         break;
      else;
         progressdlg([it/curr_ini.maxit max_grad]);
      end;
   end;
   i_b = kron(eye(n),B_0_inv);
   info_mat = 2*T*S'*i_b'*Nn*i_b*S;
   %
%   if rank(info_mat)<size(info_mat,1);
%      if showwaitbar==1;
%         set(mywait,'UserData','done');
%         close(mywait);
%         drawnow;
%      end;
%      rank_test = rank([2*i_b'*Nn*i_b;Rmat]);
%      return;
%   end;
   change = info_mat\grad;
   if max(abs(change))>=1;
      change = step*(1/max(abs(change)))*change;
   else;
      change = step*change;
   end;
   gamma = gamma+change;
   B_0 = vecinv(S*gamma+d,n);
   if rank(B_0)<n;
      if showwaitbar==1;
         set(mywait,'UserData','done');
         close(mywait);
         drawnow;
      end;
      rank_test = rank([2*i_b'*Nn*i_b;Rmat]);
      return;
   end;
   B_0_inv = B_0\eye(n);
   grad = -T*S'*kron(B_0_inv,B_0_inv')*(vec(B_0')-( kron(Omega,eye(n))*vec(B_0_inv) ) );
   max_grad = max(abs(grad));
end;

if showwaitbar==1;
   set(mywait,'UserData','done');
   close(mywait);
   drawnow;
end;

%
% check if we should fix the signs of the shocks
%

H = change_ir_signs(tm.signshock,B_0,n);

B_0 = B_0*H;
B_0_inv = B_0\eye(n);
gamma = ((S'*S)\S')*kron(H',eye(n))*S*gamma;

infomat = [it max_grad];
%status = 0;
i_b = kron(eye(n),B_0_inv);
%
% test for local identification for estimated B_0
%
test_mat = [2*i_b'*Nn*i_b;Rmat];
if rank(test_mat,eps)==floor(tm.n*tm.n);
   status = 0;
end;
rank_test = rank(test_mat,eps);

info_B0 = 2*S'*i_b'*Nn*i_b*S;
Sigma_B0 = (1/T)*(S/info_B0)*S';

if (local_status==2)&(status==0);
   status = 2;
end;

%
% end of vlaar.m
%
